package cn.iocoder.yudao.module.ecw.controller.admin.dock;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.module.ecw.dal.dataobject.region.RegionDO;
import cn.iocoder.yudao.module.ecw.service.region.RegionService;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import java.util.stream.Collectors;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.ecw.controller.admin.dock.vo.*;
import cn.iocoder.yudao.module.ecw.dal.dataobject.dock.DockDO;
import cn.iocoder.yudao.module.ecw.convert.dock.DockConvert;
import cn.iocoder.yudao.module.ecw.service.dock.DockService;

@Api(tags = "管理后台 - 码头信息")
@RestController
@RequestMapping("/ecw/dock")
@Validated
public class DockController {

    @Resource
    private DockService dockService;
    @Resource
    private RegionService regionService;

    @PostMapping("/create")
    @ApiOperation("创建码头信息")
    @PreAuthorize("@ss.hasPermission('ecw:dock:create')")
    public CommonResult<Long> createDock(@Valid @RequestBody DockCreateReqVO createReqVO) {
        return success(dockService.createDock(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新码头信息")
    @PreAuthorize("@ss.hasPermission('ecw:dock:update')")
    public CommonResult<Boolean> updateDock(@Valid @RequestBody DockUpdateReqVO updateReqVO) {
        dockService.updateDock(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除码头信息")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:dock:delete')")
    public CommonResult<Boolean> deleteDock(@RequestParam("id") Long id) {
        dockService.deleteDock(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得码头信息")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:dock:query')")
    public CommonResult<DockRespVO> getDock(@RequestParam("id") Long id) {
        DockDO dock = dockService.getDock(id);
        matchRegion(Collections.singletonList(dock));
        return success(DockConvert.INSTANCE.convert(dock));
    }

    @GetMapping("/list")
    @ApiOperation("获得码头信息列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
//    @PreAuthorize("@ss.hasPermission('ecw:dock:query')")
    public CommonResult<List<DockRespVO>> getDockList(@RequestParam("ids") Collection<Long> ids) {
        List<DockDO> list = dockService.getDockList(ids);
        matchRegion(list);
        return success(DockConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得码头信息分页")
//    @PreAuthorize("@ss.hasPermission('ecw:dock:query')")
    public CommonResult<PageResult<DockRespVO>> getDockPage(@Valid DockPageReqVO pageVO) {
        PageResult<DockDO> pageResult = dockService.getDockPage(pageVO);
        matchRegion(pageResult.getList());
        return success(DockConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出码头信息 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:dock:export')")
    @OperateLog(type = EXPORT)
    public void exportDockExcel(@Valid DockExportReqVO exportReqVO,
              HttpServletResponse response) throws IOException {
        List<DockDO> list = dockService.getDockList(exportReqVO);
        matchRegion(list);
        // 导出 Excel
        List<DockExcelVO> datas = DockConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "码头信息.xls", "数据", DockExcelVO.class, datas);
    }

    //关联查询最新的国家省市区
    private void matchRegion(List<DockDO> list) {
        if(CollectionUtil.isNotEmpty(list)) {
            List<Long> regionIdList = new ArrayList<>();
            for (DockDO dockDO : list) {
                if(dockDO.getCountryId() != null) {
                    regionIdList.add(dockDO.getCountryId());
                }
                if(dockDO.getProvinceId() != null) {
                    regionIdList.add(dockDO.getProvinceId());
                }
                if(dockDO.getCityId() != null) {
                    regionIdList.add(dockDO.getCityId());
                }
            }
            List<RegionDO> regionList = regionService.getRegionList(regionIdList);
            Map<Long, RegionDO> regionMap = regionList.stream()
                    .collect(Collectors.toMap(RegionDO::getId, v->v, (v1,v2)->v2));
            for (DockDO dockDO : list) {
                Long countryId = dockDO.getCountryId();
                if(countryId != null && regionMap.containsKey(countryId)) {
                    RegionDO country = regionMap.get(countryId);
                    dockDO.setCountryZh(country.getTitleZh());
                    dockDO.setCountryEn(country.getTitleEn());
                }
                Long provinceId = dockDO.getProvinceId();
                if(provinceId != null && regionMap.containsKey(provinceId)) {
                    RegionDO province = regionMap.get(provinceId);
                    dockDO.setProvinceZh(province.getTitleZh());
                    dockDO.setProvinceEn(province.getTitleEn());
                }
                Long cityId = dockDO.getCityId();
                if(cityId != null && regionMap.containsKey(cityId)) {
                    RegionDO city = regionMap.get(cityId);
                    dockDO.setCityZh(city.getTitleZh());
                    dockDO.setCityEn(city.getTitleEn());
                }
            }
        }
    }

}
