package cn.iocoder.yudao.module.ecw.controller.app.region;

import cn.iocoder.yudao.framework.apollo.core.event.QueryChannelInfoEvent;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.module.ecw.controller.admin.region.vo.*;
import cn.iocoder.yudao.module.ecw.convert.region.RegionConvert;
import cn.iocoder.yudao.module.ecw.dal.dataobject.region.RegionDO;
import cn.iocoder.yudao.module.ecw.service.region.RegionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.enums.GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Api(tags = "app-web - 国家城市管理")
@RestController
@RequestMapping("/ecw/region")
@Validated
public class AppRegionController {

    @Autowired
    private RegionService regionService;
    @Resource
    private ApplicationContext applicationContext;

    @GetMapping("/get")
    @ApiOperation("获得区域设置")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:region:query')")
    public CommonResult<RegionRespVO> getRegion(@RequestParam("id") Long id) {
        RegionDO region = regionService.getRegion(id);
        return success(RegionConvert.INSTANCE.convert(region));
    }



    @GetMapping("/list")
    @ApiOperation("获得区域设置列表")
    @PreAuthenticated
    public CommonResult<List<RegionRespVO>> getRegionList(RegionListReqVO reqVO) {
        List<RegionDO> list = regionService.getSimpleDepts(reqVO);
        list.sort(Comparator.comparing(RegionDO::getSort));
        return success(RegionConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得区域设置分页")
    @PreAuthenticated
    public CommonResult<PageResult<RegionRespVO>> getRegionPage(@Valid RegionPageReqVO pageVO) {
        PageResult<RegionDO> pageResult = regionService.getRegionPage(pageVO);
        return success(RegionConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/getTree")
    @ApiOperation("获得区域子列表")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthenticated
    public CommonResult<RegionRespVO> getTree(@RequestParam("id") Long id) {
        List<RegionDO> list = regionService.getByParent(id);
        List<RegionRespVO> dataList = RegionConvert.INSTANCE.convertList(list);
        Optional<RegionRespVO> optional = dataList.stream().filter(t -> t.getId().equals(id)).findFirst();
        if(!optional.isPresent()) {
            return error(INTERNAL_SERVER_ERROR.getCode(), "数据不存在");
        }
        RegionRespVO regionRespVO =  optional.get();
        buildChildTree(regionRespVO, dataList);
        return success(regionRespVO);
    }

    @GetMapping("/getListTree")
    @ApiOperation("获得区域设置列表")
    @PreAuthenticated
    public CommonResult<List<RegionRespVO>> listTree(RegionListReqVO reqVO) {
        List<RegionDO> list = regionService.getSimpleDepts(reqVO);
        list.sort(Comparator.comparing(RegionDO::getSort));
        List<RegionRespVO> dataList = RegionConvert.INSTANCE.convertList(list);

        List<RegionRespVO> continents = dataList.stream()
                .filter(t -> t.getParentId() == 0)
                .collect(Collectors.toList());
        for (RegionRespVO regionRespVO : continents) {
            buildChildTree(regionRespVO, dataList);
        }
        if(reqVO.getTreeType() == 0) {
            return success(continents);
        }
        List<RegionRespVO> countryList = continents.stream()
                .map(RegionRespVO::getChildren)
                .flatMap(Collection::stream)
                .collect(Collectors.toList());

        return success(countryList);
    }

    @GetMapping("/getCityList")
    @ApiOperation("获得区域城市列表")
    @PreAuthenticated
    public CommonResult<List<RegionRespVO>> getCityList() {
        List<RegionDO> cityList = regionService.getCityList();
        return success(RegionConvert.INSTANCE.convertList(cityList));
    }

    @GetMapping("/getTradeCityList")
    @ApiOperation("获得始发、目的城市列表")
    @PreAuthenticated
    public CommonResult<List<RegionRespVO>> getTradeCityList(TradeRegionListReqVO reqVO) {
        if (Objects.nonNull(reqVO.getChannelId())){
            // 如果有渠道信息，则查询可出目的地国家下的目的地城市
            QueryChannelInfoEvent channelInfoEvent = new QueryChannelInfoEvent();
            channelInfoEvent.setChannelId(reqVO.getChannelId());
            applicationContext.publishEvent(channelInfoEvent);
            if (Objects.nonNull(channelInfoEvent.getCountryId()) && channelInfoEvent.getCountryId() > 0L){
                reqVO.setDestCountryId(channelInfoEvent.getCountryId());
            }
        }
        List<RegionDO> cityList = regionService.getImportExportCityList(reqVO);
        return success(RegionConvert.INSTANCE.convertList(cityList));
    }

    private RegionRespVO buildChildTree(RegionRespVO pNode, List<RegionRespVO> allDatas){
        List<RegionRespVO> childs = new  ArrayList<>();
        for(RegionRespVO itemNode : allDatas) {
            if(itemNode.getParentId().equals(pNode.getId())) {
                childs.add(buildChildTree(itemNode, allDatas));
            }
        }
        pNode.setChildren(childs);
        return pNode;
    }

}
