package cn.iocoder.yudao.module.ecw.service.supplier;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.ecw.controller.admin.supplier.vo.supplierExternal.SupplierExternalCreateReqVO;
import cn.iocoder.yudao.module.ecw.controller.admin.supplier.vo.supplierResource.SupplierResourceCreateReqVO;
import cn.iocoder.yudao.module.ecw.convert.supplier.*;
import cn.iocoder.yudao.module.ecw.dal.dataobject.supplier.*;
import cn.iocoder.yudao.module.ecw.dal.mysql.supplier.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import java.text.DecimalFormat;
import java.util.*;
import java.util.stream.Collectors;

import cn.iocoder.yudao.module.ecw.controller.admin.supplier.vo.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.*;

/**
 * 供应商 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class SupplierServiceImpl implements SupplierService {

    @Resource
    private SupplierMapper supplierMapper;

    @Resource
    private SupplierBankMapper supplierBankMapper;

    @Resource
    private SupplierContactsMapper supplierContactsMapper;

    @Resource
    private SupplierResourceMapper supplierResourceMapper;

    @Resource
    private SupplierExternalMapper supplierExternalMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createSupplier(SupplierCreateReqVO createReqVO) {
        // 插入
        SupplierDO supplier = SupplierConvert.INSTANCE.convert(createReqVO);
        if(StringUtils.isEmpty(createReqVO.getCompanyType())) {
            supplier.setCompanyType("");
        }
        supplierMapper.insert(supplier);

        Long supplierId = supplier.getId();

        DecimalFormat decimalFormat = new DecimalFormat("0000");
        String companyCode = "GYS" + decimalFormat.format(supplierId);
        supplier.setCompanyCode(companyCode);
        supplierMapper.updateById(supplier);

        List<SupplierBankBaseVO> bankList = createReqVO.getBankList();
        if(CollectionUtil.isNotEmpty(bankList)) {
            List<SupplierBankDO> bankVoList = new ArrayList<>();
            for (SupplierBankBaseVO supplierBankBaseVO : bankList) {
                SupplierBankDO supplierBankDO = SupplierBankConvert.INSTANCE.convert(supplierBankBaseVO);
                supplierBankDO.setSupplierId(supplierId);
                bankVoList.add(supplierBankDO);
            }
            supplierBankMapper.insertBatch(bankVoList);
        }

        List<SupplierContactsBaseVO> contactList = createReqVO.getContactList();
        if(CollectionUtil.isNotEmpty(contactList)) {
            List<SupplierContactsDO> contactsVoList = new ArrayList<>();
            for (SupplierContactsBaseVO supplierContactsBaseVO : contactList) {
                SupplierContactsDO supplierContactsDO = SupplierContactsConvert.INSTANCE.convert(supplierContactsBaseVO);
                supplierContactsDO.setSupplierId(supplierId);
                contactsVoList.add(supplierContactsDO);
            }
            supplierContactsMapper.insertBatch(contactsVoList);
        }

        SupplierResourceCreateReqVO resourceVO = createReqVO.getResourceVO();
        if(createReqVO.getAreaType() == 0 && resourceVO != null) {
            //国内
            SupplierResourceDO supplierResourceDO = SupplierResourceConvert.INSTANCE.convert(resourceVO);
            supplierResourceDO.setSupplierId(supplierId);
            supplierResourceMapper.insert(supplierResourceDO);
        }

        SupplierExternalCreateReqVO externalVO = createReqVO.getExternalVO();
        if(createReqVO.getAreaType() == 1 && externalVO != null) {
            //国外
            SupplierExternalDO supplierExternalDO = SupplierExternalConvert.INSTANCE.convert(externalVO);
            supplierExternalDO.setSupplierId(supplierId);
            supplierExternalMapper.insert(supplierExternalDO);
        }

        // 返回
        return supplier.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateSupplier(SupplierUpdateReqVO updateReqVO) {
        Long supplierId = updateReqVO.getId();
        // 校验存在
        SupplierDO oldSupplierDO = this.validateSupplierExists(supplierId);
        // 更新
        SupplierDO updateObj = SupplierConvert.INSTANCE.convert(updateReqVO);
        if(!oldSupplierDO.getAreaType().equals(updateObj.getAreaType())) {
            throw new ServiceException(500, "供应商类型不能修改");
        }
        supplierMapper.updateById(updateObj);

        List<SupplierBankBaseVO> bankList = updateReqVO.getBankList();
        List<SupplierContactsBaseVO> contactList = updateReqVO.getContactList();
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("supplier_id", supplierId);
        if(CollectionUtil.isEmpty(bankList)) {
            //删掉银行
            supplierBankMapper.deleteByMap(paramMap);
        } else {
            //差异性处理
            //获取数据库已存在的银行
            List<SupplierBankDO> saveBankList = new ArrayList<>();
            List<SupplierBankDO> updateBankList = new ArrayList<>();

            for (SupplierBankBaseVO supplierBankBaseVO : bankList) {
                Long bankId = supplierBankBaseVO.getId();
                SupplierBankDO bankDO = SupplierBankConvert.INSTANCE.convert(supplierBankBaseVO);
                if(bankId != null) {
                    updateBankList.add(bankDO);
                } else {
                    saveBankList.add(bankDO);
                }
                bankDO.setSupplierId(supplierId);
            }
            //删除掉不在更新id列表的数据
            if(CollectionUtil.isEmpty(updateBankList)) {
                //没有更新的，删除所有老的
                supplierBankMapper.deleteByMap(paramMap);
            } else {
                List<Long> updateBankIdList = updateBankList.stream()
                        .map(SupplierBankDO::getId)
                        .collect(Collectors.toList());
                supplierBankMapper.delete(new LambdaQueryWrapperX<SupplierBankDO>()
                        .notIn(CollectionUtil.isNotEmpty(updateBankIdList), SupplierBankDO::getId, updateBankIdList));
            }

            //更新已存在或者新增的数据
            supplierBankMapper.insertBatch(saveBankList);
            supplierBankMapper.updateBatch(updateBankList);
        }

        if(CollectionUtil.isEmpty(contactList)) {
            //删掉联系人
            supplierContactsMapper.deleteByMap(paramMap);
        } else {
            //差异性处理
            //获取数据库已存在的银行
            List<SupplierContactsDO> saveContactList = new ArrayList<>();
            List<SupplierContactsDO> updateContactList = new ArrayList<>();

            for (SupplierContactsBaseVO supplierContactsBaseVO : contactList) {
                Long contactId = supplierContactsBaseVO.getId();
                SupplierContactsDO contactsDO = SupplierContactsConvert.INSTANCE.convert(supplierContactsBaseVO);
                if(contactId != null) {
                    updateContactList.add(contactsDO);
                } else {
                    saveContactList.add(contactsDO);
                }
                contactsDO.setSupplierId(supplierId);
            }
            //删除掉不在更新id列表的数据
            if(CollectionUtil.isEmpty(updateContactList)) {
                //没有更新的，删除所有老的
                supplierContactsMapper.deleteByMap(paramMap);
            } else {
                List<Long> updateContactIdList = updateContactList.stream()
                        .map(SupplierContactsDO::getId)
                        .collect(Collectors.toList());
                supplierContactsMapper.delete(new LambdaQueryWrapperX<SupplierContactsDO>()
                        .notIn(CollectionUtil.isNotEmpty(updateContactIdList), SupplierContactsDO::getId, updateContactIdList));
            }

            //更新已存在或者新增的数据
            supplierContactsMapper.insertBatch(saveContactList);
            supplierContactsMapper.updateBatch(updateContactList);
        }

        //更新国内供应商信息
        SupplierResourceCreateReqVO resourceVO = updateReqVO.getResourceVO();
        if(updateReqVO.getAreaType() == 0 && resourceVO != null) {
            //国内
            SupplierResourceDO dbDO = supplierResourceMapper.selectOne("supplier_id", supplierId);
            SupplierResourceDO supplierResourceDO = SupplierResourceConvert.INSTANCE.convert(resourceVO);
            if(dbDO != null) {
                supplierResourceDO.setId(dbDO.getId());
                supplierResourceMapper.updateById(supplierResourceDO);
            } else {
                supplierResourceMapper.insert(supplierResourceDO);
            }
        }

        //更新国外供应商信息
        SupplierExternalCreateReqVO externalVO = updateReqVO.getExternalVO();
        if(updateReqVO.getAreaType() == 1 && externalVO != null) {
            SupplierExternalDO dbDO = supplierExternalMapper.selectOne("supplier_id", supplierId);
            SupplierExternalDO supplierExternalDO = SupplierExternalConvert.INSTANCE.convert(externalVO);
            if(dbDO != null) {
                supplierExternalDO.setId(dbDO.getId());
                supplierExternalMapper.updateById(supplierExternalDO);
            } else {
                supplierExternalMapper.insert(supplierExternalDO);
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteSupplier(Long id) {
        // 校验存在
        this.validateSupplierExists(id);
        // 删除
        supplierMapper.deleteById(id);
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("supplier_id", id);
        supplierBankMapper.deleteByMap(paramMap);
        supplierContactsMapper.deleteByMap(paramMap);
        supplierResourceMapper.deleteByMap(paramMap);
        supplierExternalMapper.deleteByMap(paramMap);
    }

    private SupplierDO validateSupplierExists(Long id) {
        SupplierDO supplierDO = supplierMapper.selectById(id);
        if (supplierDO == null) {
            throw exception(SUPPLIER_NOT_EXISTS);
        }
        return supplierDO;
    }

    @Override
    public SupplierDO getSupplier(Long id) {
        return supplierMapper.selectById(id);
    }

    @Override
    public SupplierRespVO getSupplierDetail(Long id) {
        SupplierDO supplier = supplierMapper.selectById(id);
        SupplierRespVO respVO = SupplierConvert.INSTANCE.convert(supplier);
        if(respVO == null) throw exception(SUPPLIER_NOT_EXISTS);

        List<SupplierBankDO> bankDOList = supplierBankMapper.selectList(id);
        List<SupplierBankBaseVO> bankBaseVOList = SupplierBankConvert.INSTANCE.convertList(bankDOList);
        respVO.setBankList(bankBaseVOList);

        List<SupplierContactsDO> contactsDOList = supplierContactsMapper.selectList(id);
        List<SupplierContactsBaseVO> contactsRespVOList = SupplierContactsConvert.INSTANCE.convertList(contactsDOList);
        respVO.setContactList(contactsRespVOList);

        SupplierResourceDO resourceDO = supplierResourceMapper.selectOne("supplier_id", id);
        respVO.setResourceBackVO(SupplierResourceConvert.INSTANCE.convert(resourceDO));

        SupplierExternalDO externalDO = supplierExternalMapper.selectOne("supplier_id", id);
        respVO.setExternalBackVO(SupplierExternalConvert.INSTANCE.convert(externalDO));
        return respVO;
    }

    @Override
    public List<SupplierDO> getSupplierList(Collection<Long> ids) {
        return supplierMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<SupplierDO> getSupplierPage(SupplierPageReqVO pageReqVO) {
        return supplierMapper.selectPage(pageReqVO);
    }

    @Override
    public void matchExtraInfo(PageResult<SupplierRespVO> pageResult) {
        List<SupplierRespVO> respList =  pageResult.getList();
        if(CollectionUtil.isNotEmpty(respList)) {
            for (SupplierRespVO respVO : respList) {
                Long id = respVO.getId();
                SupplierResourceDO resourceDO = supplierResourceMapper.selectOne("supplier_id", id);
                respVO.setResourceBackVO(SupplierResourceConvert.INSTANCE.convert(resourceDO));

                SupplierExternalDO externalDO = supplierExternalMapper.selectOne("supplier_id", id);
                respVO.setExternalBackVO(SupplierExternalConvert.INSTANCE.convert(externalDO));
            }
        }
    }

    @Override
    public List<SupplierDO> getSupplierList(SupplierExportReqVO exportReqVO) {
        return supplierMapper.selectList(exportReqVO);
    }


    //--------------------------------提单托运人----------------------------

    @Override
    public Long createLadingShipper(LadingShipperCreateReqVO createReqVO) {
        SupplierDO supplier = SupplierConvert.INSTANCE.convert(createReqVO);
        supplier.setType(1);
        supplier.setCompanyType("");
        supplier.setCooperationType(0);
        supplierMapper.insert(supplier);
        return supplier.getId();
    }

    @Override
    public void updateLadingShipper(LadingShipperUpdateReqVO updateReqVO) {
        Long supplierId = updateReqVO.getId();
        // 校验存在
        this.validateSupplierExists(supplierId);
        // 更新
        SupplierDO updateObj = SupplierConvert.INSTANCE.convert(updateReqVO);
        supplierMapper.updateById(updateObj);
    }

    @Override
    public void deleteLadingShipper(Long id) {
        // 校验存在
        this.validateSupplierExists(id);
        // 删除
        supplierMapper.deleteById(id);
    }

    @Override
    public PageResult<SupplierDO> getLadingShipperPage(LadingShipperPageReqVO pageReqVO) {
        return supplierMapper.selectLadingShipperPage(pageReqVO);
    }

    @Override
    public List<SupplierDO> getLadingShipperList(LadingShipperExportReqVO exportReqVO) {
        return supplierMapper.selectList(exportReqVO);
    }
}
